@@ -1,9 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 """SPF (Sender Policy Framework) implementation.
 
 Copyright (c) 2003, Terence Way
-Portions Copyright (c) 2004,2005,2006,2007 Stuart Gathman <stuart@bmsi.com>
-Portions Copyright (c) 2005,2006,2007 Scott Kitterman <scott@kitterman.com>
+Portions Copyright (c) 2004,2005,2006,2007,2008 Stuart Gathman <stuart@bmsi.com>
+Portions Copyright (c) 2005,2006,2007,2008 Scott Kitterman <scott@kitterman.com>
 This module is free software, and you may redistribute it and/or modify
 it under the same terms as Python itself, so long as this copyright message
 and disclaimer are retained in their original form.
@@ -28,8 +28,44 @@ For news, bugfixes, etc. visit the home page for this implementation at
     http://www.wayforward.net/spf/
 """
 
-# CVS Commits since last release (2.0.3):
+# CVS Commits since last release (2.0.4):
 # $Log: spf.py,v $
+# Revision 1.108.2.33  2008/04/23 21:00:42  customdesigned
+# Quote nulls in Received-SPF.
+#
+# Revision 1.108.2.32  2008/04/23 20:03:53  customdesigned
+# Add timeout keyword to query constructor and DNSLookup.
+#
+# Revision 1.108.2.31  2008/03/27 01:15:33  customdesigned
+# Improve valid DNS name check.
+#
+# Revision 1.108.2.30  2008/03/27 00:58:15  customdesigned
+# Check dns names before DNSLookup
+#
+# Revision 1.108.2.29  2008/03/26 15:08:20  kitterma
+# Fix commit log typo.
+#
+# Revision 1.108.2.28  2008/03/26 14:45:37  kitterma
+# Update built in tests for Python2.5 (addr2bin will now fail slightly with older
+# Python versions). SF #1655736
+#
+# Revision 1.108.2.27  2008/03/26 14:34:35  kitterma
+# Change shebangs to #!/usr/bin/python throughout.
+#
+# Revision 1.108.2.26  2008/03/26 14:31:04  kitterma
+# Patch from Debian to avoid crash if command line SPF record request returns
+# TempError or PermError.
+#
+# Revision 1.108.2.25  2008/03/26 14:26:19  kitterma
+# Update for new version (working on 2.0.5) and year.
+#
+# Revision 1.108.2.24  2008/03/24 21:33:22  customdesigned
+# Patch from Scott Kitterman to retry truncated results with TCP unless
+# in harsh mode.
+#
+# Revision 1.108.2.23  2007/11/28 19:48:37  customdesigned
+# Reflect decision on empty-exp errata.
+#
 # Revision 1.108.2.22  2007/06/23 20:17:09  customdesigned
 # Don't try to include null (None) keyword values.
 #
@@ -49,15 +85,11 @@ For news, bugfixes, etc. visit the home page for this implementation at
 # Revision 1.108.2.17  2007/03/13 20:13:16  customdesigned
 # Missing parentheses.
 #
-# Revision 1.108.2.16  2007/01/25 20:50:13  kitterma
-# Update versions to reflect working on 2.0.4 now.
-#
-#
 # See pyspf_changelog.txt for earlier CVS commits.
 
 __author__ = "Terence Way"
 __email__ = "terry@wayforward.net"
-__version__ = "2.0.4: January 24, 2007"
+__version__ = "2.0.5: March 26, 2008"
 MODULE = 'spf'
 
 USAGE = """To check an incoming mail request:
@@ -89,15 +121,24 @@ if not hasattr(DNS.Type, 'SPF'):
     DNS.Type.typemap[99] = 'SPF'
     DNS.Lib.RRunpacker.getSPFdata = DNS.Lib.RRunpacker.getTXTdata
 
-def DNSLookup(name, qtype, strict=True):
+def DNSLookup(name, qtype, strict=True, timeout=30):
     try:
-        req = DNS.DnsRequest(name, qtype=qtype)
+        req = DNS.DnsRequest(name, qtype=qtype, timeout=timeout)
         resp = req.req()
         #resp.show()
         # key k: ('wayforward.net', 'A'), value v
         # FIXME: pydns returns AAAA RR as 16 byte binary string, but
         # A RR as dotted quad.  For consistency, this driver should
         # return both as binary string.
+        #
+        if resp.header['tc'] == True:
+          if strict > 1:
+              raise TempError, 'DNS: Truncated UDP Reply, not retrying TCP'
+          try:
+              req = DNS.DnsRequest(name, qtype=qtype, protocol='tcp')
+              resp = req.req()
+          except DNS.DNSError, x:
+              raise TempError, 'DNS: TCP Fallback error: ' + str(x)
         return [((a['name'], a['typename']), a['data']) for a in resp.answers]
     except IOError, x:
         raise TempError, 'DNS ' + str(x)
@@ -295,7 +336,8 @@ class query(object):
 
     Also keeps cache: DNS cache.  
     """
-    def __init__(self, i, s, h, local=None, receiver=None, strict=True):
+    def __init__(self, i, s, h, local=None, receiver=None, strict=True,
+                timeout=30):
         self.s, self.h = s, h
         if not s and h:
             self.s = 'postmaster@' + h
@@ -320,6 +362,7 @@ class query(object):
         self.lookups = 0
         # strict can be False, True, or 2 (numeric) for harsh
         self.strict = strict
+        self.timeout = timeout
         if i:
             self.set_ip(i)
         self.default_modifier = True
@@ -727,11 +770,16 @@ class query(object):
             modifiers.append(mod)
             if mod == 'exp':
                 # always fetch explanation to check permerrors
+                if not arg:
+                    raise PermError('exp has empty domain-spec:',arg)
                 arg = self.expand_domain(arg)
-                exp = self.get_explanation(arg)
-                if exp and not recursion:
-                    # only set explanation in base recursion level
-                    self.set_explanation(exp)
+                if arg:
+                    try:
+                        exp = self.get_explanation(arg)
+                        if exp and not recursion:
+                            # only set explanation in base recursion level
+                            self.set_explanation(exp)
+                    except: pass
             elif mod == 'redirect':
                 self.check_lookups()
                 redirect = self.expand_domain(arg)
@@ -1142,12 +1190,14 @@ class query(object):
         pre: qtype in ['A', 'AAAA', 'MX', 'PTR', 'TXT', 'SPF']
         post: isinstance(__return__, types.ListType)
         """
+        if not reduce(lambda x,y:x and 0 < len(y) < 64, name.split('.'),True):
+            return []   # invalid DNS name (too long or empty)
         result = self.cache.get( (name, qtype) )
         cname = None
 
         if not result:
             safe2cache = query.SAFE2CACHE
-            for k, v in DNSLookup(name, qtype, self.strict):
+            for k, v in DNSLookup(name, qtype, self.strict, self.timeout):
                 if k == (name, 'CNAME'):
                     cname = v
                 if (qtype,k[1]) in safe2cache:
@@ -1293,7 +1343,8 @@ def quote_value(s):
     """
     if s is None or RE_DOT_ATOM.match(s):
       return s
-    return '"' + s.replace('\\',r'\\').replace('"',r'\"') + '"'
+    return '"' + s.replace('\\',r'\\').replace('"',r'\"'
+                ).replace('\x00',r'\x00') + '"'
 
 def parse_mechanism(str, d):
     """Breaks A, MX, IP4, and PTR mechanisms into a (name, domain,
@@ -1390,7 +1441,7 @@ def addr2bin(str):
 
     Examples::
     >>> addr2bin('127.0.0.1')
-    2130706433L
+    2130706433
 
     >>> addr2bin('127.0.0.1') == socket.INADDR_LOOPBACK
     1
@@ -1404,12 +1455,12 @@ def addr2bin(str):
     Unlike DNS.addr2bin, the n, n.n, and n.n.n forms for IP addresses
     are handled as well::
     >>> addr2bin('10.65536')
-    167837696L
+    167837696
     >>> 10 * (2 ** 24) + 65536
     167837696
 
     >>> addr2bin('10.93.512')
-    173867520L
+    173867520
     >>> 10 * (2 ** 24) + 93 * (2 ** 16) + 512
     173867520
     """
@@ -1616,9 +1667,14 @@ if __name__ == '__main__':
         print USAGE
         _test()
     elif len(sys.argv) == 2:
-        q = query(i='127.0.0.1', s='localhost', h='unknown',
-            receiver=socket.gethostname())
-        print q.dns_spf(sys.argv[1])
+        try:
+            q = query(i='127.0.0.1', s='localhost', h='unknown',
+                receiver=socket.gethostname())
+            print q.dns_spf(sys.argv[1])
+        except TempError, x:
+            print "Temporary DNS error: ", x
+        except PermError, x:
+            print "PermError: ", x
     elif len(sys.argv) == 4:
         print check(i=sys.argv[1], s=sys.argv[2], h=sys.argv[3],
             receiver=socket.gethostname())
